package cm.bound;

import cm.bound.admin.AdminServer;
import cm.bound.configuration.Configuration;
import cm.bound.configuration.ConfigurationBuilder;
import cm.bound.configuration.options.BasicOption;
import cm.bound.configuration.options.Option;
import cm.utils.L10N;
import cm.utils.Logging;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * $Rev: 105 $ <br/>
 * $Date: 2005-04-22 04:07:58 +0900 (Fri, 22 Apr 2005) $ <br/>
 * $Author: C.M $
 * <p/>
 * Bound CNXB
 */
public class Bound {
  private static final L10N L10N = new L10N("cm.bound.Bound");

  /*
  * Oo̓CjV
  */
  private void initLogger(BasicOption basicOp) throws IOException{
    Logging sysLog = new Logging("system_log");
    sysLog.setLevel(Logging.INFO);
    sysLog.addAppender(new FileWriter(basicOp.getSystemLog(),true));
    if(basicOp.isVerbose()) sysLog.setLevel(Logging.DEBUG);
    Log.setSysLogger(sysLog);

    Logging accessLog = new Logging("access_log");
    accessLog.setLevel(Logging.INFO);
    accessLog.addAppender(new FileWriter(basicOp.getAccessLog(),true));
    if(basicOp.isVerbose()) accessLog.setLevel(Logging.DEBUG);
    Log.setAccessLogger(accessLog);

    Logging consoleLog = new Logging("console_log");
    consoleLog.setLevel(Logging.INFO);
    consoleLog.addAppender(new PrintWriter(System.out));
    if(basicOp.isVerbose()) consoleLog.setLevel(Logging.DEBUG);
    Log.setConsoleLogger(consoleLog);
  }

  private static void exit(Throwable t){
    if(t != null){
      String msg = L10N.getString("exitSystemWithProblemError");
      if(Log.getSysLogger() != null && Log.getConsoleLogger() != null){
        Log.getSysLogger().error(msg,t);
        Log.getConsoleLogger().error(msg,t);
      }else{
        System.err.println(msg);
        t.printStackTrace();
      }
      System.exit(1);
    }
    System.exit(0);
  }

   public static void main(String[] localOps) {
    Bound bound = new Bound();
    try{
      Configuration config = ConfigurationBuilder.buildConfiguration(localOps);
      if(config == null) exit(null); // have been showed help or version

      bound.initLogger(config.getBasicOption());
      String version = L10N.getString("showVersionInfo", config.getBasicOption().getVersion());
      Log.getConsoleLogger().info(version);
      Log.getSysLogger().info(version);
      String copyright = config.getBasicOption().getCopyright();
      Log.getConsoleLogger().info(copyright);
      Log.getSysLogger().info(copyright);

      int adminPort = config.getBasicOption().getAdminPort();
      if(adminPort > 0){
        new AdminServer(config.getBasicOption().getAdminPort()).start();
      }

      int ops = config.getOptions().size();
      for(int i = 0; i < ops; i++){
        Option op = (Option)config.getOptions().get(i);
        new Server(op).start();
      }
    }catch(Throwable t){
      exit(t);
    }
  }
}
