package cm.bound;

import cm.bound.configuration.options.Option;
import cm.io.DumpOutputStream;
import cm.io.GZIPPackInputStream;
import cm.io.GZIPPackOutputStream;
import cm.io.HttpOutputStream;
import cm.io.StreamProperty;
import cm.utils.L10N;
import cm.utils.StreamUtil;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.SocketException;

/**
 * $Rev: 82 $ <br/>
 * $Date: 2005-04-20 22:53:22 +0900 (Wed, 20 Apr 2005) $ <br/>
 * $Author: C.M $
 * <p/>
 * cm.bound.Request NXBpPbg[{B
 * @see cm.bound.Request
 */
public class RequestImpl implements Request {
  private static final L10N L10N = new L10N("cm.bound.RequestImpl");

  public static final int TCP  = Option.TCP;
  public static final int SSL  = Option.SSL;
  public static final int HTTP = Option.HTTP;
  public static final int GZIP = Option.GZIP;
  public static final int DUMP = Option.DUMP;

  private byte[] _buf = new byte[StreamProperty.BUFFER_SIZE];
  private RequestOption _ro;

  /**
  * w肵 ro  RequestImpl 쐬B
  * @param ro NGXgpIvVB
  */
  public RequestImpl(RequestOption ro) {
    _ro = ro;
  }

  private OutputStream getOutputStream() throws IOException{
    OutputStream ret;
    switch(_ro.getToProtocol()){
      case GZIP:
        ret = new GZIPPackOutputStream(_ro.getToSocket().getOutputStream());
        break;
      case HTTP:
        ret = new BufferedOutputStream(
                new HttpOutputStream(_ro.getToSocket(), _ro.getHttpHeaders(), _ro.getDelHttpHeaders()));
        break;
      case DUMP:
        ret = new BufferedOutputStream(
                new DumpOutputStream(_ro.getToSocket().getOutputStream(), _ro.getDumper()));
        break;
      default:
        ret = new BufferedOutputStream(_ro.getToSocket().getOutputStream());
    }
    return ret;
  }

  private InputStream getInputStream() throws IOException{
    InputStream ret;
    switch(_ro.getFromProtocol()){
      case GZIP:
        ret = new GZIPPackInputStream(_ro.getFromSocket().getInputStream());
        break;
      case HTTP:
        ret = new BufferedInputStream(_ro.getFromSocket().getInputStream());
        break;
      case DUMP:
        ret = new BufferedInputStream(_ro.getFromSocket().getInputStream());
        break;
      default:
        ret = new BufferedInputStream(_ro.getFromSocket().getInputStream());
    }
    return ret;
  }

  /**
  * pPbg[{B
  */
  public void execute() {
    InputStream in = null;
    OutputStream out = null;
    try {
      in = getInputStream();
      out = getOutputStream();
      int c = 0;
      //ȉpPbg[ƂȂB
      while((c=in.read()) != -1) {
        out.write(c);
        send(in, out);
      }
    }catch(SocketException e) {
      // \Pbg̃N[Y͐ȓȂ̂ŁAȊÕG[L^B
      String error = e.getMessage();
      if(error == null || !(error.endsWith(" closed")
                            || error.endsWith("Connection reset")
                            || error.endsWith("Broken pipe"))){
        Log.getSysLogger().error(this, e);
      }
    }catch(IOException e){
      // Xg[̃N[Y̓\Pbg̃N[Yɂ̂Ȃ̂ŁAȊÕG[L^B
      String error = e.getMessage();
      if(error == null || !error.endsWith(" closed")){
        Log.getSysLogger().error(this, e);
      }
    }catch(Exception e){
      Log.getSysLogger().error(this, e);
    } finally {
      try {
        String msg = L10N.getString("closedConectionInfo", _ro.getFromSocket().toString());
        Log.getAccessLogger().info(msg);
        if(Log.getConsoleLogger().isDebugEnabled()){
          Log.getConsoleLogger().debug(msg);
        }
        if(in != null) in.close();
        if(out != null) out.close();
        _ro.getToSocket().close();
        _ro.getFromSocket().close();
      }catch(Exception e){
        Log.getSysLogger().error(this, e);
      }
    }
  }

  private void send(InputStream from, OutputStream to) throws IOException{
    int available = from.available();
    if(available > 0){
      int count = available / _buf.length;
      int tailDataLen = available % _buf.length;
      int b;
      for(int i=0; i<count; i++) {
        b = from.read(_buf, 0, _buf.length);
        if(b!=-1) {
          to.write(_buf, 0, b);
        } else {
          Log.getSysLogger().warn(L10N.getString("streamClosedWhileSendingPacktsWarn"));
          break;
        }
      }
      b = from.read(_buf, 0, tailDataLen);
      if(b!=-1){
        to.write(_buf, 0, b);
      }else{
        Log.getSysLogger().warn(L10N.getString("streamClosedWhileSendingPacktsWarn"));
      }
    }
    to.flush();
    if(Log.getConsoleLogger().isDebugEnabled()){
      Log.getConsoleLogger().debug(" >> Flushed " + available + " byes");
    }
  }

  /**
   * pPbg[ɕKvȏێNXB
   */
  public static class RequestOption {
    private int _fromProtocol;
    private int _toProtocol;
    private Socket _fromSocket;
    private Socket _toSocket;
    private OutputStream _dumper;
    private String[] _toHttpHeaders;
    private String[] _delToHttpHeaders;

    public RequestOption(Socket from, int frprotcl, Socket to, int toprotcl){
      _fromSocket = from;
      _toSocket = to;
      _fromProtocol = frprotcl;
      _toProtocol = toprotcl;
    }

    public void setDumper(OutputStream dumper){
      _dumper = dumper;
    }
    public OutputStream getDumper(){
      return _dumper;
    }

    public int getFromProtocol(){
      return _fromProtocol;
    }
    public int getToProtocol(){
      return _toProtocol;
    }

    public Socket getFromSocket(){
      return _fromSocket;
    }
    public Socket getToSocket(){
      return _toSocket;
    }

    public void setHttpHeaders(String[] headers){
    _toHttpHeaders = headers;
    }
    public String[] getHttpHeaders(){
      return _toHttpHeaders;
    }

    public void setDelHttpHeaders(String[] headers){
      _delToHttpHeaders = headers;
    }
    public String[] getDelHttpHeaders(){
      return _delToHttpHeaders;
    }
  }
}
