package cm.bound;

import cm.utils.L10N;
import java.util.ArrayList;
import java.util.LinkedList;

/**
 * $Rev: 59 $ <br/>
 * $Date: 2005-03-02 19:58:40 +0900 (Wed, 02 Mar 2005) $ <br/>
 * $Author: C.M $
 * <p/>
 * 󂯕tNGXg𗭂߂L[\NXB
 * ɃNGXgL[ƃ`lXbh̃v[ێB
 */
public class RequestQueue
{
  private static final L10N L10N = new L10N("cm.bound.RequestQueue");

  private final LinkedList _requestQueue = new LinkedList(); //NGXgL[
  private final ArrayList _channelPool = new ArrayList(); //`lXbhv[
  private final ArrayList _channelTrashBin = new ArrayList(); //`lXbhݔ
  private final int _maxQueueLen;
  private final int _initChannelCount;
  private final int _maxChannelCount;
  private final int _gainChannelCount;

  /**
  * w肵 maxQueueLen ̃NGXgL[Ǝw肵 channelCount 
  * cm.bound.Channel ̃IuWFNg̃Xbhv[ێ RequestQueue 쐬B
  * @param maxQueueLen NGXgL[̍ő咷
  * @param initChannelCount `lXbh̐̏l
  * @param maxChannelCount `lXbh̐̍ől
  * @param gainChannelCount `lXbh̐̑
  * @see cm.bound.Channel
  */
  public RequestQueue(int maxQueueLen, int initChannelCount, int maxChannelCount, int gainChannelCount){
    _maxQueueLen = maxQueueLen;
    _initChannelCount = initChannelCount;
    _maxChannelCount = maxChannelCount;
    _gainChannelCount = gainChannelCount;
    for(int i=0; i<_initChannelCount; i++){
      _channelPool.add(new Channel(this));
    }
  }

  /**
  * `lXbhv[̃XbhSċNB
  * @see cm.bound.Channel
  */
  public void startChannels(){
    int len = _channelPool.size();
    for(int i=0; i<len; i++){
      Channel c = (Channel)_channelPool.get(i);
      c.start();
      if(Log.getConsoleLogger().isDebugEnabled()){
        Log.getConsoleLogger().debug("channel thread #" + i + " started up");
      }
    }
  }

  /**
  * `lXbĥݔw肵`lXbh폜B
  * @param c ݔ폜`lXbhB
  * @return 폜ꂽꍇ trueA폜Ȃꍇ falseB
  * @see cm.bound.Channel
  */
  public boolean removeChannel(Channel c){
    return _channelTrashBin.remove(c);
  }

  /**
  * w肵NGXg request NGXgL[̐擪ɒuB
  * @param request NGXgL[ɒu Request IuWFNgB
  * @see cm.bound.Request
  * @exception NGXgL[ɂςꍇB
  */
  public synchronized void putRequest(Request request) throws RejectRequestException{
    if(_requestQueue.size() >= _maxQueueLen){
      throw new RejectRequestException(L10N.getString("requestQueueSizeReachedMaxLen"));
    }
    _requestQueue.addFirst(request);
    if(_requestQueue.size() * 10 > _maxQueueLen * 9){
      String msg = L10N.getString("requestQueueSizIsEmarginal");
      Log.getSysLogger().warn(msg);
      Log.getConsoleLogger().warn(msg);
    }
    if(Log.getConsoleLogger().isDebugEnabled()){
      Log.getConsoleLogger().debug("A new request was put in the request queue. queue size: " + _requestQueue.size());
    }
    updateChannelPool();
    notifyAll();
  }

  private void updateChannelPool(){
    int busy = busyChannelsCount();
    int nowlen = _channelPool.size();
    if(Log.getConsoleLogger().isDebugEnabled()){
      Log.getConsoleLogger().debug("busy Channels: " + busy + ", " + "all Channels: " + nowlen);
    }
    //݂̃`lXbĥA90%ȏオrW[Xbh𑝂₷
    if(nowlen * 9 < busy * 10){
      if(increaseThreadPool(busy, nowlen) > 0){
        String[] s = new String[3];
        s[0] = String.valueOf(busy);
        s[1] = String.valueOf(nowlen);
        s[2] = String.valueOf(_channelPool.size());
        String msg = L10N.getString("increaseChannelThread", s);
        Log.getSysLogger().info(msg);
        if(Log.getConsoleLogger().isDebugEnabled()){
          Log.getConsoleLogger().debug(msg);
        }
      }else{
        String msg = L10N.getString("reachedMaxChannelThreads", String.valueOf(_maxChannelCount));
        Log.getSysLogger().warn(msg);
        if(Log.getConsoleLogger().isDebugEnabled()){
          Log.getConsoleLogger().debug(msg);
        }
      }
    }else if(_initChannelCount < nowlen){ //`lXbh𑝉Ăꍇ
      //rW[̃`lXbhÃ݂Xbh30%ȉɂȂꍇAeʕXbh炷
      if(busy * 10 <= nowlen * 3){
        decreaseThreadPool(busy, nowlen);
        String[] s = new String[3];
        s[0] = String.valueOf(busy);
        s[1] = String.valueOf(nowlen);
        s[2] = String.valueOf(_channelPool.size());
        String msg = L10N.getString("decreaseChannelThread", s);
        Log.getSysLogger().info(msg);
        if(Log.getConsoleLogger().isDebugEnabled()){
          Log.getConsoleLogger().debug(msg);
        }
      }
    }
  }

  private int increaseThreadPool(int busy, int nowlen){
    int count = 0;
    for(int i = 0; i < _gainChannelCount; i++){
      if(_maxChannelCount < 0){ // _maxChannelCount  0 菬疳ő
        Channel c = new Channel(this);
        c.start();
        _channelPool.add(c);
        count += 1;
      }else{
        if(_channelPool.size() < _maxChannelCount){
          Channel c = new Channel(this);
          c.start();
          _channelPool.add(c);
          count += 1;
        }
      }
    }
    return count;
  }

  private int decreaseThreadPool(int busy, int nowlen){
    int count = 0;
    int newlen = nowlen;
    int stoplen = nowlen - _gainChannelCount;
    if(stoplen < _initChannelCount){
       stoplen = _initChannelCount;
    }
    for(int i = nowlen -1 ; i >= 0; i--){
      Channel c = (Channel)_channelPool.get(i);
      if(!c.busy()){ //rW[łȂXbhݔɈړ
        c.setContinue(false);
        _channelTrashBin.add(c);
        _channelPool.remove(i);
        newlen -= 1;
        count += 1;
        if(newlen == stoplen)break;
      }
    }
    return count;
  }

  private int busyChannelsCount(){
    int len = _channelPool.size();
    int count = 0;
    for(int i=0; i<len; i++){
      Channel c = (Channel)_channelPool.get(i);
      if(c.busy()){
        count += 1;
      }
    }
    return count;
  }

  /**
  * NGXgL[̍Ō烊NGXgIuWFNgoB
  * @return NGXgL[o Request IuWFNgB
  * @see cm.bound.Request
  */
  public synchronized Request pullRequest(){
    while(_requestQueue.size() <= 0){
      try{
        wait();
      }catch(InterruptedException e){}
    }
    Request ret = (Request)_requestQueue.removeLast();
    if(Log.getConsoleLogger().isDebugEnabled()){
      Log.getConsoleLogger().debug("The channel thread taked up the request form the request queue. queue size: " + _requestQueue.size());
    }
    return ret;
  }

}
