package cm.bound;

import cm.bound.RequestImpl.RequestOption;
import cm.bound.configuration.options.Option;
import cm.bound.handler.CheckSocketHandler;
import cm.bound.handler.HandlerException;
import cm.bound.handler.OpenSocketHandler;
import cm.bound.handler.SSLSocketHandler;
import cm.bound.handler.SocketHandlerChain;
import cm.bound.handler.TunnelSocketHandler;
import cm.security.DefaultKey;
import cm.security.FileKey;
import cm.security.Key;
import cm.utils.EncodeOutputStream;
import cm.utils.L10N;
import cm.utils.SocketUtil;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.security.AccessControlException;
import javax.net.ServerSocketFactory;
import javax.net.ssl.SSLServerSocket;

/**
 * $Rev: 105 $ <br/>
 * $Date: 2005-04-22 04:07:58 +0900 (Fri, 22 Apr 2005) $ <br/>
 * $Author: C.M $
 * <p/>
 * pPbgpp҂󂯃T[oXbh\NXB
 */
public class Server extends Thread{
  private static final L10N L10N = new L10N("cm.bound.Server");

  private final static int BACKLOG = 100; //T[o\Pbg̃obOOB
  private static int _clientCount; // ڑNCAgB
  private static int _dumpNum = 0; //_vԍB
  private static char[] _dumpFileTail = {'0','0','0','0','0','0','0','0','0'};//_vt@Cɕt镶B

  private static long _acceptCount; //ڑ accept 
  private static long _rejectCount; //ڑ reject 

  /**
  * ݂܂Őڑ擾B
  * @return ݂܂ŐڑB
  */
  public static long getAcceptCount() {
    return _acceptCount;
  }

  /**
  * ݂܂Őڑۂ擾B
  * @return ݂܂ŐڑۂB
  */
  public static long getRejectCount() {
    return _rejectCount;
  }

  private final Option _op;
  private RequestQueue _rqOutbound; //AEgoEhNGXgL[
  private RequestQueue _rqInbound; //CoEhNGXgL[

  /**
  * op Ŏw肳ꂽRtBO[VŁARun 쐬B
  * @param op pXbhɑ΂IvVB
  */
  public Server(Option op){
    _op = op;
     initRequestQueue();
  }
  /*
  * NGXgL[CjV
  */
  private void initRequestQueue(){
    int qlen = _op.getQueueLength();
    int initThreads = _op.getInitThreads();
    int maxThreads = _op.getMaxThreads();
    int gainThreads = _op.getGainThreads();

    String[] s = new String[5];
    s[0] = L10N.getString("outbound");
    s[1] = String.valueOf(qlen);
    s[2] = String.valueOf(initThreads);
    if(maxThreads < 0){
      s[3] = "unlimited";
    }else{
      s[3] = String.valueOf(maxThreads);
    }
    s[4] = String.valueOf(gainThreads);
    _rqOutbound = new RequestQueue(qlen, initThreads, maxThreads, gainThreads);
    _rqOutbound.startChannels();
    String info = L10N.getString("startWorkerThreadsInfo", s);
    Log.getConsoleLogger().info(info);
    Log.getSysLogger().info(info);

    s[0] = L10N.getString("inbound");
    _rqInbound = new RequestQueue(qlen, initThreads, maxThreads, gainThreads);
    _rqInbound.startChannels();
    info = L10N.getString("startWorkerThreadsInfo", s);
    Log.getConsoleLogger().info(info);
    Log.getSysLogger().info(info);
  }

  public void run() {
    int listenPort = _op.getListenPort();
    String[] inboundHttpHeaders = _op.getInboundHttpHeaders();
    String[] delInboundHttpHeaders = _op.getDelInboundHttpHeaders();
    String[] outboundHttpHeaders = _op.getOutboundHttpHeaders();
    String[] delOutboundHttpHeaders = _op.getDelOutboundHttpHeaders();
    int outboundProtocol = _op.getOutboundProtocol();
    int inboundProtocol = _op.getInboundProtocol();
    String forwardHost = _op.getForwardHost();
    int forwardPort = _op.getForwardPort();
    String proxyHost = _op.getProxyHost();
    int proxyPort = _op.getProxyPort();
    String proxyAuth = _op.getProxyAuth();
    OpenSocketHandler openSocketHandler;
    int dumpFormat = _op.getDumpFormat();
    String dumpLogDir = _op.getDumpLogDir();
    if(proxyHost != null){
      openSocketHandler = new OpenSocketHandler(proxyHost, proxyPort);
    }else{
      openSocketHandler = new OpenSocketHandler(forwardHost, forwardPort);
    }
    Key serverKey = new DefaultKey();
    if(_op.getServerKeyStore() != null){
      serverKey = new FileKey("JKS",
             _op.getServerKeyStore(),
             _op.getServerStorePass(),
             _op.getServerKeyPass());
    }
    Key clientKey = new DefaultKey();
    if(_op.getClientKeyStore() != null){
      clientKey = new FileKey("JKS",
             _op.getClientKeyStore(),
             _op.getClientStorePass(),
             _op.getClientKeyPass());
    }
    try {
      ServerSocketFactory ssFactory;
      if(inboundProtocol == Option.SSL){
        ssFactory = SocketUtil.getServerSocketFactory(serverKey);
      }else{
        ssFactory = SocketUtil.getServerSocketFactory(null);
      }
      ServerSocket serverSocket = ssFactory.createServerSocket(listenPort, BACKLOG);
      if(_op.isNeedClientAuth()){
        ((SSLServerSocket)serverSocket).setNeedClientAuth(true);
      }

      printLaunchLog(listenPort, forwardHost, forwardPort,
        inboundProtocol, outboundProtocol, proxyHost, proxyPort);

      while(true) {
        Socket client = null;
        Socket server = null;
        try {
          client = serverSocket.accept();
          if(client!=null) {
            String msg = L10N.getString("acceptConnectionInfo", client.toString());
            Log.getAccessLogger().info(msg);
            if(Log.getConsoleLogger().isDebugEnabled()){
              Log.getConsoleLogger().debug(msg);
            }

            //ڑւ̃\Pbgnh`FC̍쐬
            SocketHandlerChain clientChain = new SocketHandlerChain(client);
            clientChain.addHandler(new CheckSocketHandler(_op.getAllowAddresses()));

            client = clientChain.doConnect();
            client.setTcpNoDelay(true);
            //ڑւ̃\Pbgnh`FC̍쐬
            SocketHandlerChain serverChain = new SocketHandlerChain();
            serverChain.addHandler(openSocketHandler);
            if(proxyHost != null){
              serverChain.addHandler(new TunnelSocketHandler(forwardHost, forwardPort, proxyAuth));
            }
            if(outboundProtocol == Option.SSL){
              serverChain.addHandler(
                new SSLSocketHandler(forwardHost, forwardPort, clientKey, _op.isIgnoreServerAuth()));
            }
            server = serverChain.doConnect();
            server.setTcpNoDelay(true);

            RequestOption outboundRO
             = new RequestOption(client, inboundProtocol, server, outboundProtocol);
            RequestOption inboundRO
             = new RequestOption(server, outboundProtocol, client, inboundProtocol);

            if(outboundProtocol == Option.HTTP){
              outboundRO.setHttpHeaders(outboundHttpHeaders);
              outboundRO.setDelHttpHeaders(delOutboundHttpHeaders);
            }
            if(inboundProtocol == Option.HTTP){
              inboundRO.setHttpHeaders(inboundHttpHeaders);
              inboundRO.setDelHttpHeaders(delInboundHttpHeaders);
            }
            if(inboundProtocol == Option.DUMP || outboundProtocol == Option.DUMP){
              synchronized(Server.class){
                _dumpNum++;
                char[] tmp = Integer.toString(_dumpNum).toCharArray();
                int i = tmp.length - 1;
                int j = _dumpFileTail.length - 1;
                if(i <= j){ // ANZX9񐔈ȏɂȂ_vȂ
                  while(i >= 0){
                    _dumpFileTail[j] = tmp[i];
                    i--;
                    j--;
                  }
                  String fileTail = new String(_dumpFileTail);
                  if(inboundProtocol == Option.DUMP){
                    inboundRO.setDumper(getDumpOutputStream(dumpFormat, dumpLogDir, "inbound"));
                  }
                  if(outboundProtocol == Option.DUMP){
                    outboundRO.setDumper(getDumpOutputStream(dumpFormat, dumpLogDir, "outbound"));
                  }
                }
              }
            }

            Request outbound = new RequestImpl(outboundRO);
            _rqOutbound.putRequest(outbound);
            Request inbound = new RequestImpl(inboundRO);
            _rqInbound.putRequest(inbound);
            String[] s = new String[2];
            s[0] = forwardHost;
            s[1] = String.valueOf(forwardPort);
            msg = L10N.getString("openConnectionToInfo", s);
            Log.getAccessLogger().info(msg);
            if(Log.getConsoleLogger().isDebugEnabled()){
              Log.getConsoleLogger().debug(msg);
            }
            _acceptCount++;
          }
        } catch(Exception e) {
          if(e instanceof RejectRequestException ||
             e instanceof AccessControlException||
             e instanceof HandlerException){
            Log.getAccessLogger().warn(L10N.getString("rejectRequestWarn"), e);
            _rejectCount++;
          }else{
            String error = L10N.getString("serverThreadGotError", String.valueOf(listenPort));
            Log.getSysLogger().error(error, e);
            Log.getConsoleLogger().error(error,e);
          }
          if(client != null) client.close();
          if(server != null) server.close();
        }
      }
    } catch(Exception e) {
      String error = L10N.getString("serverThreadGotError", String.valueOf(listenPort));
      Log.getSysLogger().error(error, e);
      Log.getConsoleLogger().error(error,e);
    }
  }

  private OutputStream getDumpOutputStream(int dumpFormat, String dumpLogDir, String basename) throws Exception{
    String fileTail = new String(_dumpFileTail);
    StringBuffer logFile = new StringBuffer();
    logFile.append(dumpLogDir);
    logFile.append(File.separator);
    logFile.append(basename);
    logFile.append(fileTail);
    logFile.append(".log");
    return new BufferedOutputStream(
             new EncodeOutputStream(
               new FileOutputStream(
                 new File(logFile.toString())), dumpFormat));
  }

  private void printLaunchLog(int listenPort, String forwardHost, int forwardPort,
                              int inboundProtocol, int outboundProtocol,
                              String proxyHost, int proxyPort
                              ){
    String sarrow = L10N.getString("tcpArrow");
    String _LINE_SEP = System.getProperty("line.separator");
    if(inboundProtocol == Option.SSL)  sarrow = L10N.getString("sslArrow");
    else if(inboundProtocol == Option.GZIP) sarrow = L10N.getString("gzipArrow");
    else if(inboundProtocol == Option.HTTP) sarrow = L10N.getString("httpArrow");
    else if(inboundProtocol == Option.DUMP) sarrow = L10N.getString("dumpArrow");
    String carrow = L10N.getString("tcpArrow");
    if(outboundProtocol == Option.SSL)  carrow = L10N.getString("sslArrow");
    else if(outboundProtocol == Option.GZIP) carrow = L10N.getString("gzipArrow");
    else if(outboundProtocol == Option.HTTP) carrow = L10N.getString("httpArrow");
    else if(outboundProtocol == Option.DUMP) carrow = L10N.getString("dumpArrow");
    String info = L10N.getString("enteredListenModeByInfo");
    StringBuffer sb = new StringBuffer();
    sb.append(info);
    sb.append(_LINE_SEP);
    sb.append(" "); sb.append(sarrow); sb.append(" "); sb.append(listenPort);
    if(proxyHost != null){
       sb.append(" "); sb.append(carrow); sb.append(" ");
       sb.append(proxyHost); sb.append(":"); sb.append(proxyPort);
       sb.append(" "); sb.append(carrow); sb.append(" ");
       sb.append(forwardHost); sb.append(":"); sb.append(forwardPort);
    }else{
       sb.append(" "); sb.append(carrow);  sb.append(" ");
       sb.append(forwardHost); sb.append(":"); sb.append(forwardPort);
    }
    Log.getConsoleLogger().info(sb);
    Log.getSysLogger().info(sb);
  }
}
