package cm.bound.admin;

import cm.bound.Version;
import cm.bound.admin.command.Command;
import cm.bound.admin.command.ShutdownCommand;
import cm.bound.admin.command.VersionCommand;
import cm.bound.configuration.ConfigurationException;
import cm.utils.L10N;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import org.apache.commons.cli.BasicParser;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;

/**
 * $Rev: 82 $ <br/>
 * $Date: 2005-04-20 22:53:22 +0900 (Wed, 20 Apr 2005) $ <br/>
 * $Author: C.M $
 * <p/>
 * ͂AǗR}h̃RtBO[V쐬NXB̃NX̃IuWFNg𓾂B̕@
 * getInstance \bhĂԂƁBIuWFNg擾́AeRtBO[V getter
 * \bhłɎ擾\B
 */
public class Configuration
{
  private static final L10N L10N = new L10N("cm.bound.admin.Configuration");

  public static final String COMM_VERSION  = "VERSION";
  public static final String COMM_SHUTDOWN = "SHUTDOWN";

  private static final String HELP         = "h";
  private static final String EXAMPLE      = "e";
  private static final String HOST         = "host";
  private static final String PORT         = "port";

  private static final String _version = Version.getVersion();
  private static final String _copyright = Version.getCopyright();
  private String _host;
  private int _port;
  private Command _command;

  public int getPort(){
    return _port;
  }
  public String getHost(){
    return _host;
  }
  public Command getCommand(){
    return _command;
  }

  private static final HelpFormatter HF = new HelpFormatter();

  private Configuration(){}

  private static void printHelp(Options options, boolean example){
     String[] replace = new String[2];
     replace[0] = _version;
     replace[1] = _copyright;
     HF.printHelp(999,
                  L10N.getString("usageHeader1"),
                  L10N.getString("usageHeader2"),
                  options,
                  L10N.getString("usageHeader3", replace),
                  false);
     if(example){
       System.out.print(L10N.getString("askExample"));
       try{
         BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
         String yes = br.readLine();
         if(yes.charAt(0) == 'y' || yes.charAt(0) == 'Y'){
           printExample();
         }
         br.close();
       }catch(Exception ignore){}
     }
  }

  private static void printExample(){
     System.out.println();
     System.out.println(L10N.getString("example1"));
     System.out.println();
     System.out.println(L10N.getString("example2"));
     System.out.println();
  }

  private static void printHelp(Options options){
    printHelp(options, false);
  }

/**
 * w肵 args ͂AConfiguration IuWFNg쐬B
 * @param args ͂B
 * @exception sꍇɃX[B
 */
  public static Configuration getInstance(String[] args) throws ConfigurationException{
    Options options = buildOptions();
    for(int i =0; i<args.length; i++){
      if(args[i].equals("-h") || args[i].equals("--help")){
          printHelp(options, true);
          return null;
      }
      if(args[i].equals("-e") || args[i].equals("--example")){
          printExample();
          return null;
      }
    }
    try{
      CommandLineParser parser = new BasicParser();
      CommandLine cl = parser.parse(options, args);
      Configuration config = new Configuration();

      config._host = cl.getOptionValue(HOST);
      config._port = Integer.parseInt(cl.getOptionValue(PORT));

      String[] largs = cl.getArgs();
      if(largs.length < 1){
        printHelp(options);
        throw new ConfigurationException(L10N.getString("commandNotFoundError"));
      }
      //CR}h
      String mainComm = largs[0];
      if(mainComm.equalsIgnoreCase(COMM_VERSION)){
        config._command = new VersionCommand();
      }else if(mainComm.equalsIgnoreCase(COMM_SHUTDOWN)){
        config._command = new ShutdownCommand();
      }else{
        printHelp(options);
        throw new ConfigurationException(L10N.getString("unsupportedCommandError", mainComm));
      }
      return config;
    }catch(MissingOptionException moe){
      printHelp(options);
      throw new ConfigurationException(L10N.getString("missingOptionError", moe.getMessage()));
    }catch(MissingArgumentException mae){
      printHelp(options);
      throw new ConfigurationException(L10N.getString("missingArgumentError", mae.getMessage()));
    }catch(UnrecognizedOptionException uoe){
      printHelp(options);
      throw new ConfigurationException(L10N.getString("unrecognizedOptionError", uoe.getMessage()));
    }catch(ParseException pe){
      printHelp(options);
      throw new ConfigurationException(L10N.getString("parseError"), pe);
    }
  }

  /*
  * IvV̍쐬B
  */
  private static Options buildOptions(){
    Options options = new Options();
    options.addOption(OptionBuilder
                      .hasArg(false)
                      .isRequired(false)
                      .withLongOpt("help")
                      .withDescription(L10N.getString("helpDescription"))
                      .create(HELP));

    options.addOption(OptionBuilder
                      .hasArg(false)
                      .isRequired(false)
                      .withLongOpt("example")
                      .withDescription(L10N.getString("exampleDescription"))
                      .create(EXAMPLE));

    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(true)
                      .withArgName("HOSTNAME")
                      .withDescription(L10N.getString("hostDescription"))
                      .create(HOST));

    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(true)
                      .withArgName("PORT")
                      .withDescription(L10N.getString("portDescription"))
                      .create(PORT));

    return options;
  }
}
