package cm.bound.admin;

import cm.bound.admin.command.Command;
import cm.bound.admin.command.CommandResult;
import cm.bound.handler.CheckSocketHandler;
import cm.bound.handler.OpenSocketHandler;
import cm.bound.handler.SocketHandlerChain;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
/**
 * $Rev: 42 $ <br/>
 * $Date: 2005-02-12 01:22:34 +0900 (Sat, 12 Feb 2005) $ <br/>
 * $Author: C.M $
 * <p/>
 * ǗR}hCNX
 */
public class Main
{
  private Socket _bound;

  private void connectToBound(String host, int port) throws Exception{
    OpenSocketHandler openSocketHandler
     = new OpenSocketHandler(host, port);
    SocketHandlerChain chain = new SocketHandlerChain();
    chain.addHandler(openSocketHandler);
    _bound = chain.doConnect();
    _bound.setTcpNoDelay(true);
  }

  private void sendCommand(Command comm) throws Exception{
    ObjectOutputStream oos = new ObjectOutputStream(_bound.getOutputStream());
    oos.writeObject(comm);
    oos.flush();
  }

  private CommandResult getCommandResult() throws Exception{
    ObjectInputStream ois = new ObjectInputStream(_bound.getInputStream());
    return (CommandResult)ois.readObject();
  }

  private void printResult(CommandResult cr){
    int returnCode = cr.getReturnCode();
    String description = cr.getDescription();
    Exception exception = cr.getException();
    if(description != null){
      System.out.println(description);
    }
    if(exception != null){
      System.out.println("Exception : ");
      exception.printStackTrace();
    }
    System.out.println("Return code : " + returnCode);
  }

  private void cleanUp(){
    try{
      if(_bound != null) _bound.close();
    }catch(Exception e){
      e.printStackTrace();
    }
  }

  public static void main(String[] args) {
    Main me = new Main();
    try{
      Configuration config = Configuration.getInstance(args);
      if(config == null) System.exit(0); // have been showed help or version

      me.connectToBound(config.getHost(), config.getPort());
      me.sendCommand(config.getCommand());
      CommandResult cr = me.getCommandResult();
      me.printResult(cr);
    }catch(Exception e){
      e.printStackTrace();
    }finally{
      me.cleanUp();
    }
  }
}
