package cm.bound.configuration;

import cm.bound.configuration.options.BasicOption;
import cm.bound.configuration.options.Option;
import cm.utils.L10N;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;

/**
 * $Rev: 106 $ <br/>
 * $Date: 2005-04-22 04:09:48 +0900 (Fri, 22 Apr 2005) $ <br/>
 * $Author: C.M $
 * <p/>
 * ͂ABound ̃RtBO[V쐬NXB
 */
public class ConfigurationBuilder
{
  private static final L10N L10N = new L10N("cm.bound.configuration.ConfigurationBuilder");

  private static final HelpFormatter HF = new HelpFormatter();

  private ConfigurationBuilder(){}

  /**
   * w肵 args ͂AConfiguration IuWFNg쐬B
   * @param args ͂B
   * @exception sꍇɃX[B
  */
  public static Configuration buildConfiguration(String[] args) throws ConfigurationException{
    for(int i =0; i<args.length; i++){
      if(args[i].equals("-h") || args[i].equals("--help")){
          printHelp(true);
          return null;
      }
      if(args[i].equals("-e") || args[i].equals("--example")){
          printExample();
          return null;
      }
      if(args[i].equals("-v") || args[i].equals("--version")){
          System.out.println(BasicOption.getVersion());
          return null;
      }
    }
    Configuration config = Configuration.getInstance();

    ArrayList l = new ArrayList();
    ArrayList ops = new ArrayList();
    for(int i=0; i<args.length; i++){
      if(args[i].startsWith("-")){
        l.add(args[i]);
      }else{
        //FIXME: check: pT[o쐬̃IvVǂ̃`FbN
        int check = args[i].split(":").length;
        if(!l.isEmpty() && check == 3){
          ops.add(l.toArray());
          l.clear();
        }
        l.add(args[i]);
      }
      if(i == args.length - 1){
         ops.add(l.toArray());
      }
    }

    try{
      int s = ops.size();
      BasicOption basicOp = new BasicOption();
      for(int i=0; i<s; i++){
        Object[] o = (Object[])ops.get(i);
        String[] arg = new String[o.length];
        for(int j=0; j<arg.length; j++){
          arg[j] = (String)o[j];
        }
        if(arg[0].startsWith("-")){ //Global Option
          basicOp.setup(arg);
        }else{
          Option op = new Option();
          op.setup(arg);
          config.addOption(op);
        }
      }
      config.setBasicOption(basicOp);
      if(config.getOptions().isEmpty()){
        printHelp();
        throw new ConfigurationException(L10N.getString("noArgumentsError"));
      }
      return config;
    }catch(MissingOptionException moe){
      printHelp();
      throw new ConfigurationException(L10N.getString("missingOptionError", moe.getMessage()));
    }catch(MissingArgumentException mae){
      printHelp();
      throw new ConfigurationException(L10N.getString("missingArgumentError", mae.getMessage()));
    }catch(UnrecognizedOptionException uoe){
      printHelp();
      throw new ConfigurationException(L10N.getString("unrecognizedOptionError", uoe.getMessage()));
    }catch(ParseException pe){
      printHelp();
      throw new ConfigurationException(L10N.getString("parseError"), pe);
    }catch(IOException e){
      throw new ConfigurationException(e);
    }
  }

  private static void printHelp(boolean example){
    PrintWriter pw = new PrintWriter(System.out, true);
     HF.printUsage(pw,
                   100,
                   L10N.getString("usageHeader1"));
     pw.println(L10N.getString("usageHeader2"));
     pw.println();
     pw.println(L10N.getString("usageHeader3"));
     Options options = BasicOption.buildOptions();
     HF.printOptions(pw,100,options,2,5);
     pw.println();
     pw.println(L10N.getString("usageHeader4"));
     options = Option.buildOptions();
     HF.printOptions(pw,100,options,2,5);
     pw.println();
     String[] replace = new String[2];
     replace[0] = BasicOption.getVersion();
     replace[1] = BasicOption.getCopyright();
     pw.println(L10N.getString("usageHeader5",replace));
    if(example){
      pw.println(L10N.getString("askExample"));
      try{
        BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
        String yes = br.readLine();
        if(yes.charAt(0) == 'y' || yes.charAt(0) == 'Y'){
          printExample();
        }
        br.close();
      }catch(Exception ignore){}
    }
  }

  private static void printExample(){
     PrintWriter pw = new PrintWriter(System.out, true);
     pw.println();
     pw.println(L10N.getString("example1"));
     pw.println();
     pw.println(L10N.getString("example2"));
     pw.println();
     pw.println(L10N.getString("example3"));
     pw.println();
     pw.println(L10N.getString("example4"));
     pw.println();
     pw.println(L10N.getString("example5"));
     pw.println();
  }

  private static void printHelp(){
    printHelp(false);
  }
}
