package cm.bound.configuration.options;

import cm.bound.Version;
import cm.utils.L10N;
import org.apache.commons.cli.BasicParser;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

/**
 * $Rev: 107 $ <br/>
 * $Date: 2005-04-22 04:14:55 +0900 (Fri, 22 Apr 2005) $ <br/>
 * $Author: C.M $
 * O[oIvV\NXB<p/>
 */
public class BasicOption{
  private static final L10N L10N = new L10N("cm.bound.configuration.options.BasicOption");

  private static final String HELP          = "h";
  private static final String EXAMPLE       = "e";
  private static final String VERSION       = "v";
  private static final String SYSTEM_LOG  = "syslog";
  private static final String ACCESS_LOG  = "aclog";
  private static final String VERBOSE     = "verbose";
  private static final String ADMINPORT   = "aport";

  private static final String _version = Version.getVersion();
  private static final String _copyright = Version.getCopyright();
  private String _syslog = "system.log";
  private String _accesslog = "access.log";
  private boolean _verbose = false;
  private int _adminport;

  /**
  * ^ꂽ args ƂɃO[oIvVݒ肷B
  * ݒ肳ꂽIvV́AsetterAgetterɂĎ擾ł悤ɂȂB
  * @param args IvV̂ƂɂȂ镶̔zB
  */
  public void setup(String[] args) throws ParseException{
    CommandLineParser parser = new BasicParser();
    CommandLine cl = parser.parse(buildOptions(), args);

    if(cl.hasOption(SYSTEM_LOG)){
      _syslog = cl.getOptionValue(SYSTEM_LOG);
    }
    if(cl.hasOption(ACCESS_LOG)){
      _accesslog = cl.getOptionValue(ACCESS_LOG);
    }
    if(cl.hasOption(VERBOSE)){
      _verbose = true;
    }
    if(cl.hasOption(ADMINPORT)){
      _adminport = Integer.parseInt(cl.getOptionValue(ADMINPORT));
    }
  }

  public String getSystemLog(){
    return _syslog;
  }
  public String getAccessLog(){
    return _accesslog;
  }
  public boolean isVerbose(){
    return _verbose;
  }
  public int getAdminPort(){
    return _adminport;
  }
  public static String getVersion(){
    return _version;
  }
  public static String getCopyright(){
    return _copyright;
  }

  /**
  * O[oIvV쐬B
  * @return O[oIvVB
  */
  public static Options buildOptions(){
    Options options = new Options();
    options.addOption(OptionBuilder
                      .hasArg(false)
                      .isRequired(false)
                      .withLongOpt("help")
                      .withDescription(L10N.getString("helpDescription"))
                      .create(HELP));

    options.addOption(OptionBuilder
                      .hasArg(false)
                      .isRequired(false)
                      .withLongOpt("example")
                      .withDescription(L10N.getString("exampleDescription"))
                      .create(EXAMPLE));

    options.addOption(OptionBuilder
                      .hasArg(false)
                      .isRequired(false)
                      .withLongOpt("version")
                      .withDescription(L10N.getString("versionDescription"))
                      .create(VERSION));

    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(false)
                      .withArgName("FILE")
                      .withLongOpt("system-log")
                      .withDescription(L10N.getString("syslogDescription"))
                      .create(SYSTEM_LOG));

    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(false)
                      .withArgName("FILE")
                      .withLongOpt("access-log")
                      .withDescription(L10N.getString("accesslogDescription"))
                      .create(ACCESS_LOG));

    options.addOption(OptionBuilder
                      .hasArg(false)
                      .isRequired(false)
                      .withLongOpt("verbose-mode")
                      .withDescription(L10N.getString("verboseDescription"))
                      .create(VERBOSE));

    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(false)
                      .withArgName("PORT")
                      .withLongOpt("admin-port")
                      .withDescription(L10N.getString("adminPortDescription"))
                      .create(ADMINPORT));

    return options;
  }
}
