package cm.bound.configuration.options;

import cm.bound.configuration.ConfigurationException;
import cm.utils.EncodeOutputStream;
import cm.utils.L10N;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import org.apache.commons.cli.BasicParser;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

/**
 * $Rev: 97 $ <br/>
 * $Date: 2005-04-21 13:16:51 +0900 (Thu, 21 Apr 2005) $ <br/>
 * $Author: C.M $
 * pXbhpIvV\NXB<p/>
 */
public class Option{
  private static final L10N L10N = new L10N("cm.bound.configuration.options.Option");

  public static final int TCP  = 0;
  public static final int SSL  = 1;
  public static final int HTTP = 2;
  public static final int GZIP = 3;
  public static final int DUMP = 4;

  private static final String ALLOW_ADDRESSES  = "a";
  private static final String QUEUE_LENGTH  = "Q";
  private static final String MAX_THREADS   = "T";
  private static final String INIT_THREADS   = "iT";
  private static final String GAIN_THREADS   = "gT";
  private static final String PROXY         = "P";
  private static final String PROXYAUTH     = "A";
  private static final String HTTP_OUTBOUND = "o";
  private static final String HTTP_INBOUND  = "i";
  private static final String DEL_HTTP_OUTBOUND = "O";
  private static final String DEL_HTTP_INBOUND  = "I";
  private static final String DUMP_LOG_DIR = "dumplog";
  private static final String DUMP_FORMAT = "dump";
  private static final String SERVER_KEYSTORE = "ss";
  private static final String SERVER_STOREPASS = "sspass";
  private static final String SERVER_KEYPASS = "skpass";
  private static final String SERVER_NEED_CLIENTAUTH = "needauth";
  private static final String CLIENT_KEYSTORE = "cs";
  private static final String CLIENT_STOREPASS = "cspass";
  private static final String CLIENT_KEYPASS = "ckpass";
  private static final String CLIENT_IGNORE_SERVERAUTH = "igauth";

  private int _listenPort = -1;
  private String _forwardHost = null;
  private String[] _allowAddresses;
  private int _queueLength = 50;
  private int _initThreads = 10;
  private int _gainThreads = 5;
  private int _maxThreads = -1;
  private int _forwardPort = -1;
  private int _inboundProtocol = TCP;
  private int _outboundProtocol = TCP;
  private String _proxyHost = null;
  private int _proxyPort = -1;
  private String _proxyAuth        = null;
  private String[] _outboundHttpHeaders = null;
  private String[] _inboundHttpHeaders = null;
  private String[] _delOutboundHttpHeaders = null;
  private String[] _delInboundHttpHeaders = null;
  private int _dumpFormat = EncodeOutputStream.BYTE;
  private String _dumplog = ".";
  private File _server_keystore = null;
  private char[] _server_storepass = null;
  private char[] _server_keypass = null;
  private boolean _need_clientauth = false;
  private File _client_keystore = null;
  private char[] _client_storepass = null;
  private char[] _client_keypass = null;
  private boolean _ignore_serverauth = false;

  public int getListenPort(){
    return _listenPort;
  }
  public String getForwardHost(){
    return _forwardHost;
  }
  public int getForwardPort(){
    return _forwardPort;
  }
  public String[] getAllowAddresses(){
    return _allowAddresses;
  }
  public int getQueueLength(){
    return _queueLength;
  }
  public int getInitThreads(){
    return _initThreads;
  }
  public int getGainThreads(){
    return _gainThreads;
  }
  public int getMaxThreads(){
    return _maxThreads;
  }
  public int getInboundProtocol(){
    return _inboundProtocol;
  }
  public int getOutboundProtocol(){
    return _outboundProtocol;
  }
  public String getProxyHost(){
    return _proxyHost;
  }
  public int getProxyPort(){
    return _proxyPort;
  }
  public String getProxyAuth(){
    return _proxyAuth;
  }
  public String[] getOutboundHttpHeaders(){
    return _outboundHttpHeaders;
  }
  public String[] getInboundHttpHeaders(){
    return _inboundHttpHeaders;
  }
  public String[] getDelOutboundHttpHeaders(){
    return _delOutboundHttpHeaders;
  }
  public String[] getDelInboundHttpHeaders(){
    return _delInboundHttpHeaders;
  }
  public int getDumpFormat(){
    return _dumpFormat;
  }
  public String getDumpLogDir(){
    return _dumplog;
  }
  public File getServerKeyStore(){
    return _server_keystore;
  }
  public char[] getServerStorePass(){
    return _server_storepass;
  }
  public char[] getServerKeyPass(){
    return _server_keypass;
  }
  public boolean isNeedClientAuth(){
    return _need_clientauth;
  }
  public File getClientKeyStore(){
    return _client_keystore;
  }
  public char[] getClientStorePass(){
    return _client_storepass;
  }
  public char[] getClientKeyPass(){
    return _client_keypass;
  }
  public boolean isIgnoreServerAuth(){
    return _ignore_serverauth;
  }

  /**
  * ^ꂽ args ƂɒpXbhpIvVݒ肷B
  * ݒ肳ꂽIvV́AsetterAgetterɂĎ擾ł悤ɂȂB
  * @param args IvV̂ƂɂȂ镶̔zB
  */
  public void setup(String[] args) throws ParseException, ConfigurationException, IOException{
    CommandLineParser parser = new BasicParser();
    CommandLine cl = parser.parse(buildOptions(), args);
    // X|[gƓ]Ɋւ̉
    if(cl.getArgs().length < 1){
      throw new ConfigurationException(L10N.getString("noListenPortAndHostError"));
    }

    String[] serv = args[0].split(":");
    if(serv.length != 3){
      throw new ConfigurationException(L10N.getString("invalidListenPortOrHostError", args[0]));
    }

    String[] p1 = serv[0].split("/");
    String[] p2 = serv[2].split("/");
    _listenPort =  Integer.parseInt(p1[0]);
    _forwardHost = serv[1];
    _forwardPort = Integer.parseInt(p2[0]);

    if(p1.length > 1){
      setupInboundProtocols(p1[1], cl);
    }
    if(p2.length > 1){
      setupOutboundProtocols(p2[1], cl);
    }
    setupCommon(cl);
  }

  private void setupCommon(CommandLine cl) throws ConfigurationException, IOException{
    String proxyHost = null;
    int proxyPort = -1;
    if(cl.hasOption(PROXY)){
      String proxyHostPort = cl.getOptionValue(PROXY);
      if(proxyHostPort.indexOf(":") <= 0){
        throw new ConfigurationException(L10N.getString("invalidProxyPortOrHostError", proxyHostPort));
      }
      proxyHost = proxyHostPort.substring(0, proxyHostPort.indexOf(":"));
      proxyPort = Integer.parseInt(proxyHostPort.substring(proxyHostPort.indexOf(":")+1, proxyHostPort.length()));
    }
    String proxyAuth = null;
    if(cl.hasOption(PROXYAUTH)){
      String auth = cl.getOptionValue(PROXYAUTH);
      ByteArrayOutputStream bao = new ByteArrayOutputStream();
      EncodeOutputStream enc = new EncodeOutputStream(bao, EncodeOutputStream.BASE64);
      enc.write(auth.getBytes());
      enc.flush();
      proxyAuth = bao.toString();
      enc.close();
    }
    if(cl.hasOption(QUEUE_LENGTH)){
      _queueLength = Integer.parseInt(cl.getOptionValue(QUEUE_LENGTH));
    }
    if(cl.hasOption(INIT_THREADS)){
      _initThreads =Integer.parseInt(cl.getOptionValue(INIT_THREADS));
    }
    if(cl.hasOption(GAIN_THREADS)){
      _gainThreads = Integer.parseInt(cl.getOptionValue(GAIN_THREADS));
    }else{
      _gainThreads = _initThreads / 2;
    }
    if(cl.hasOption(MAX_THREADS)){
      _maxThreads = Integer.parseInt(cl.getOptionValue(MAX_THREADS));
    }
    if(cl.hasOption(ALLOW_ADDRESSES)){
      _allowAddresses =  cl.getOptionValues(ALLOW_ADDRESSES);
    }
    InetAddress ad = new InetSocketAddress(_forwardHost, _forwardPort).getAddress();
    if(ad != null && (ad.isLoopbackAddress() || ad.equals(InetAddress.getLocalHost()))){
      //]悪[vobNAhX܂̓[JAhX̏ꍇ̓vLVڑȂB
    }else{
      if(proxyHost != null){
        _proxyHost = proxyHost;
        _proxyPort = proxyPort;
      }
      if(proxyAuth != null){
        _proxyAuth = proxyAuth;
      }
    }
  }

  private void setupInboundProtocols(String protocol, CommandLine cl) throws ConfigurationException{
    if(protocol.equalsIgnoreCase("ssl")){
      _inboundProtocol = Option.SSL;
      if(cl.hasOption(SERVER_KEYSTORE)){
        File keystore = new File(cl.getOptionValue(SERVER_KEYSTORE));
        if(!keystore.exists()){
          throw new ConfigurationException(L10N.getString("keyStoreDoesNotExsistError", keystore.getName()));
        }
        _server_keystore = keystore;
        if(cl.hasOption(SERVER_STOREPASS)){
          String password = cl.getOptionValue(SERVER_STOREPASS);
          _server_storepass = password.toCharArray();
        }else{
          String[] replace = new String[2];
          replace[0] = SERVER_KEYSTORE;
          replace[1] = SERVER_STOREPASS;
          throw new ConfigurationException(L10N.getString("needKeyPassOptionError", replace));
        }
        if(cl.hasOption(SERVER_KEYPASS)){
          String password = cl.getOptionValue(SERVER_KEYPASS);
          _server_keypass = password.toCharArray();
        }else{
          _server_keypass = _server_storepass;
        }
      }
      if(cl.hasOption(SERVER_NEED_CLIENTAUTH)){
        _need_clientauth = true;
      }
    }else if(protocol.equalsIgnoreCase("http")){
      String[] add = null;
      String[] del = null;
      if(cl.hasOption(HTTP_INBOUND)){
        add = cl.getOptionValues(HTTP_INBOUND);
        _inboundHttpHeaders = add;
      }
      if(cl.hasOption(DEL_HTTP_INBOUND)){
        del = cl.getOptionValues(DEL_HTTP_INBOUND);
        _delInboundHttpHeaders = del;
      }
      if(add == null && del == null){
        String[] replace = new String[2];
        replace[0] = HTTP_INBOUND;
        replace[1] = DEL_HTTP_INBOUND;
        throw new ConfigurationException(L10N.getString("needOptionsForInboundHttpProtocolError", replace));
      }
      _inboundProtocol = Option.HTTP;
    }else if(protocol.equalsIgnoreCase("gzip")){
      _inboundProtocol = Option.GZIP;
    }else if(protocol.equalsIgnoreCase("dump")){
      _inboundProtocol = Option.DUMP;
      if(cl.hasOption(DUMP_FORMAT)){
        String s = cl.getOptionValue(DUMP_FORMAT);
        if(getDumpFormat(s) == -2){
          throw new ConfigurationException(L10N.getString("nonSupportedDumpFormatError", s));
        }
        _dumpFormat = getDumpFormat(s);
      }
      if(cl.hasOption(DUMP_LOG_DIR)){
        String s = cl.getOptionValue(DUMP_LOG_DIR);
        File dir = new File(s);
        if(!dir.exists()){
          throw new ConfigurationException(L10N.getString("notExsistsDumpLogDir", s));
        }
        _dumplog = s;
      }
    }
  }

  private void setupOutboundProtocols(String protocol, CommandLine cl) throws ConfigurationException{
    if(protocol.equalsIgnoreCase("ssl")){
      _outboundProtocol = Option.SSL;
      if(cl.hasOption(CLIENT_KEYSTORE)){
        File keystore = new File(cl.getOptionValue(CLIENT_KEYSTORE));
        if(!keystore.exists()){
          throw new ConfigurationException(L10N.getString("keyStoreDoesNotExsistError", keystore.getName()));
        }
        _client_keystore = keystore;
        if(cl.hasOption(CLIENT_STOREPASS)){
          String password = cl.getOptionValue(CLIENT_STOREPASS);
          _client_storepass = password.toCharArray();
        }else{
          String[] replace = new String[2];
          replace[0] = CLIENT_KEYSTORE;
          replace[1] = CLIENT_STOREPASS;
          throw new ConfigurationException(L10N.getString("needKeyPassOptionError", replace));
        }
        if(cl.hasOption(CLIENT_KEYPASS)){
          String password = cl.getOptionValue(CLIENT_KEYPASS);
          _client_keypass = password.toCharArray();
        }else{
          _client_keypass = _client_storepass;
        }
      }
      if(cl.hasOption(CLIENT_IGNORE_SERVERAUTH)){
        _ignore_serverauth = true;
      }
    }else if(protocol.equalsIgnoreCase("http")){
      String[] add = null;
      String[] del = null;
      if(cl.hasOption(HTTP_OUTBOUND)){
        add = cl.getOptionValues(HTTP_OUTBOUND);
        _outboundHttpHeaders = add;
      }
      if(cl.hasOption(DEL_HTTP_OUTBOUND)){
        del = cl.getOptionValues(DEL_HTTP_OUTBOUND);
        _delOutboundHttpHeaders = del;
      }
      if(add == null && del == null){
        String[] replace = new String[2];
        replace[0] = HTTP_OUTBOUND;
        replace[1] = DEL_HTTP_OUTBOUND;
        throw new ConfigurationException(L10N.getString("needOptionsForOutboundHttpProtocolError", replace));
      }
      _outboundProtocol = Option.HTTP;
    }else if(protocol.equalsIgnoreCase("gzip")){
      _outboundProtocol = Option.GZIP;
    }else if(protocol.equalsIgnoreCase("dump")){
      _outboundProtocol = Option.DUMP;
      if(cl.hasOption(DUMP_FORMAT)){
        String s = cl.getOptionValue(DUMP_FORMAT);
        if(getDumpFormat(s) == -2){
          throw new ConfigurationException(L10N.getString("nonSupportedDumpFormatError", s));
        }
        _dumpFormat = getDumpFormat(s);
      }
      if(cl.hasOption(DUMP_LOG_DIR)){
        String s = cl.getOptionValue(DUMP_LOG_DIR);
        File dir = new File(s);
        if(!dir.exists()){
          throw new ConfigurationException(L10N.getString("notExsistsDumpLogDir", s));
        }
        _dumplog = s;
      }
    }
  }

  private static int getDumpFormat(String str){
    int type = -1;
    if(str.equalsIgnoreCase("byte")){
      type = EncodeOutputStream.BYTE;
    }else if(str.equalsIgnoreCase("hex")){
      type = EncodeOutputStream.HEX;
    }else if(str.equalsIgnoreCase("base64")){
      type = EncodeOutputStream.BASE64;
    }else{
      return -2;
    }
    return type;
  }

  /**
  * pXbhpIvV쐬B
  * @return pXbhpIvVB
  */
  public static Options buildOptions(){
    Options options = new Options();
    buildCommonOptions(options);
    buildProtocolOptions(options);
    return options;
  }

  private static Options buildCommonOptions(Options options){
    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(false)
                      .withArgName("HOSTNAME:PORT")
                      .withLongOpt("proxy")
                      .withDescription(L10N.getString("proxyDescription"))
                      .create(PROXY));

    options.addOption(OptionBuilder
                      .hasArgs()
                      .isRequired(false)
                      .withArgName("ADDRESSES")
                      .withLongOpt("allow-addresses")
                      .withDescription(L10N.getString("allowAddressListDescription"))
                      .create(ALLOW_ADDRESSES));

    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(false)
                      .withArgName("INTEGER")
                      .withLongOpt("max-queue-length")
                      .withDescription(L10N.getString("maxQueueLengthDescription"))
                      .create(QUEUE_LENGTH));

    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(false)
                      .withArgName("INTEGER")
                      .withLongOpt("initial-threads")
                      .withDescription(L10N.getString("initialThreadsDescription"))
                      .create(INIT_THREADS));

    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(false)
                      .withArgName("INTEGER")
                      .withLongOpt("gain-threads")
                      .withDescription(L10N.getString("gainThreadsDescription"))
                      .create(GAIN_THREADS));

    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(false)
                      .withArgName("INTEGER")
                      .withLongOpt("max-threads")
                      .withDescription(L10N.getString("maxThreadsDescription"))
                      .create(MAX_THREADS));

    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(false)
                      .withArgName("USERNAME:PASSWORD")
                      .withLongOpt("proxy-authorization")
                      .withDescription(L10N.getString("proxyAuthDescription"))
                      .create(PROXYAUTH));
    return options;
  }

  private static Options buildProtocolOptions(Options options){
    options.addOption(OptionBuilder
                      .hasArgs()
                      .isRequired(false)
                      .withArgName("STRINGS")
                      .withLongOpt("http-outbound")
                      .withDescription(L10N.getString("outboundHttpHeaderDescription"))
                      .create(HTTP_OUTBOUND));

    options.addOption(OptionBuilder
                      .hasArgs()
                      .isRequired(false)
                      .withArgName("STRINGS")
                      .withLongOpt("http-inbound")
                      .withDescription(L10N.getString("inboundHttpHeaderDescription"))
                      .create(HTTP_INBOUND));

    options.addOption(OptionBuilder
                      .hasArgs()
                      .isRequired(false)
                      .withArgName("STRINGS")
                      .withLongOpt("delete-http-outbound")
                      .withDescription(L10N.getString("delOutboundHttpHeaderDescription"))
                      .create(DEL_HTTP_OUTBOUND));

    options.addOption(OptionBuilder
                      .hasArgs()
                      .isRequired(false)
                      .withArgName("STRINGS")
                      .withLongOpt("delete-http-inbound")
                      .withDescription(L10N.getString("delInboundHttpHeaderDescription"))
                      .create(DEL_HTTP_INBOUND));

    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(false)
                      .withArgName("DIRECTORY")
                      .withLongOpt("dump-log-dir")
                      .withDescription(L10N.getString("dumplogDescription"))
                      .create(DUMP_LOG_DIR));

    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(false)
                      .withArgName("BYTE | HEX | BASE64")
                      .withLongOpt("dump-format")
                      .withDescription(L10N.getString("dumpDescription"))
                      .create(DUMP_FORMAT));

    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(false)
                      .withArgName("FILE")
                      .withLongOpt("server-keystore")
                      .withDescription(L10N.getString("sslServerKeyStoreDescription"))
                      .create(SERVER_KEYSTORE));

    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(false)
                      .withArgName("PASSWORD")
                      .withLongOpt("server-keystore-password")
                      .withDescription(L10N.getString("sslServerStorePassDescription"))
                      .create(SERVER_STOREPASS));

    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(false)
                      .withArgName("PASSWORD")
                      .withLongOpt("server-key-password")
                      .withDescription(L10N.getString("sslServerKeyPassDescription"))
                      .create(SERVER_KEYPASS));

    options.addOption(OptionBuilder
                      .hasArg(false)
                      .isRequired(false)
                      .withLongOpt("server-need-client-authentication")
                      .withDescription(L10N.getString("sslServerNeedClientAuthDescription"))
                      .create(SERVER_NEED_CLIENTAUTH));

    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(false)
                      .withArgName("FILE")
                      .withLongOpt("client-keystore")
                      .withDescription(L10N.getString("sslClientKeyStoreDescription"))
                      .create(CLIENT_KEYSTORE));

    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(false)
                      .withArgName("PASSWORD")
                      .withLongOpt("client-keystore-password")
                      .withDescription(L10N.getString("sslClientStorePassDescription"))
                      .create(CLIENT_STOREPASS));

    options.addOption(OptionBuilder
                      .hasArg(true)
                      .isRequired(false)
                      .withArgName("PASSWORD")
                      .withLongOpt("client-key-password")
                      .withDescription(L10N.getString("sslClientKeyPassDescription"))
                      .create(CLIENT_KEYPASS));

    options.addOption(OptionBuilder
                      .hasArg(false)
                      .isRequired(false)
                      .withLongOpt("client-ignore-server-authentication")
                      .withDescription(L10N.getString("sslClientIgnoreServerAuthDescription"))
                      .create(CLIENT_IGNORE_SERVERAUTH));
    return options;
  }
}
