package cm.io;

import cm.bound.Log;
import cm.utils.StreamUtil;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.Socket;
import java.util.Date;

/**
 * $Rev: 53 $ <br/>
 * $Date: 2005-03-02 00:10:14 +0900 (Wed, 02 Mar 2005) $ <br/>
 * $Author: C.M $
 * <p/>
 * HTTPɓo̓Xg[NXB
 */
public class HttpOutputStream extends FilterOutputStream implements HttpProperty
{
  private String[] _httpHeaders;
  private String[] _delHttpHeaders;
  private Socket _socket;
  private ByteArrayOutputStream _header = new ByteArrayOutputStream();
  private int _bodyRemainLength;
  private boolean _chunkedEnable;
  private StringBuffer _chunkSizeSb = new StringBuffer(4);
  private int _chunkSize;

  //wb_ҏŴ߂̃L[[h
  private final static String KW_TIME         = "%t";
  private final static String KW_DATE         = "%d";
  private final static String KW_SOCKET       = "%s";
  private final static String KW_HOST         = "%h";
  private final static String KW_PORT         = "%p";
  private final static String KW_LOCALHOST    = "%H";
  private final static String KW_LOCALPORT    = "%P";
//  private final static String KW_ACCEPT_COUNT = "%a";
//  private final static String KW_REJECT_COUNT = "%r";


  /*
   *  [START_LINE]    "GET ... HTTP/1.0" or "HTTP/1.0 STATUS CODE"   CR [START_LINE_CR] LF
   *  [START_HEADER]  "Content-Type: ... [IN_HEADER] ..."            CR [HEADER_CR]  LF
   *  [START_HEADER]  "Accept-Languate: ... [IN_HEADER] ..."         CR [HEADER_CR]  LF
   *  [START_HEADER]  CR [END_HEADER_CR] LF
   *  [START_BODY]
   *    or
   *  *[CHUNCK_SIZE] 0fe7 CR [CHUNCK_SIZE_CR] LF
   *  *[CHUNK_BODY]
   *   [START_TRAILER]  [IN_TRAILER]        CR [TRAILER_CR] LF
   *   CR LAST_CR LF
   */
  private int _status = START_LINE;
  private static final int START_LINE    = 0;
  private static final int START_LINE_CR = 1;
  private static final int START_HEADER  = 2;
  private static final int IN_HEADER     = 3;
  private static final int HEADER_CR     = 4;
  private static final int END_HEADER_CR = 5;
  private static final int START_BODY    = 6;
  private static final int CHUNK_SIZE    = 7;
  private static final int CHUNK_SIZE_CR = 8;
  private static final int CHUNK_BODY    = 9;
  private static final int START_TRAILER = 10;
  private static final int IN_TRAILER    = 11;
  private static final int TRAILER_CR    = 12;
  private static final int LAST_CR       = 13;
  private int _body_len = 0;


  /**
  * w肵 socketAǉ HTTP wb_ httpHeadersA폜 HTTP wb_ delHttpHeaders 
  * HttpOutputStream 쐬B
  * @param socket ڑp\PbgB
  * @param httpHeaders f[^Mɒǉ HTTP wb_B
  * @param delHttpHeaders f[^Mɍ폜 HTTP wb_B
  * @exception ڑɊւăG[ꍇB
  */
  public HttpOutputStream(Socket socket, String[] httpHeaders, String[] delHttpHeaders) throws IOException{
    super(socket.getOutputStream());
    _socket = socket;
    if(httpHeaders != null){
      _httpHeaders = buildHttpHeaders(httpHeaders);
    }
    _delHttpHeaders = delHttpHeaders;
  }

  /**
  * RXgN^Ŏw肵\Pbg瓾o͐Xg[ i ̃f[^ށB
  * ݂̍ہAȉlB<p/>
  * RXgN^HTTPwb_̒ǉw肵ꍇ:<br/>
  * ݎAHTTPSTART LINEIɒǉwb_ށB<p/>
  * RXgN^HTTPwb_̍폜w肵ꍇ:<br/>
  * ݃f[^폜Ώۃwb_̈ꕔ̏ꍇ͏܂ȂB
  * @param i ރoCgB
  * @exception o̓G[ꍇB
  */
  public void write(int i) throws IOException{
    switch(_status){
      case START_LINE:
        out.write(i);
        if((byte)i == CR){
          _status = START_LINE_CR;
        }
        break;
      case START_LINE_CR:
        out.write(i);
        if((byte)i == LF){
          _status = START_HEADER;
          writeHeaders();
        }
        break;
      case START_HEADER:
        if((byte)i == CR){
          out.write(i);
          _status = END_HEADER_CR;
        }else{
          _header.write(i);
          _status = IN_HEADER;
        }
        break;
      case IN_HEADER:
        _header.write(i);
        if((byte)i == CR){
          _status = HEADER_CR;
        }
        break;
      case HEADER_CR:
        _header.write(i);
        if((byte)i == LF){
          checkBodyLength();
          checkDeleteHeaders();
          _status = START_HEADER;
        }
        break;
      case END_HEADER_CR:
        out.write(i);
        if((byte)i == LF){
          if(_chunkedEnable){
             _status = CHUNK_SIZE;
          }else if(_bodyRemainLength > 0){
             _status = START_BODY;
          }else{
             _status = START_LINE;
          }
        }
        break;
      case START_BODY:
        out.write(i);
        _bodyRemainLength -= 1;
        if(_bodyRemainLength == 0){
          _status = START_LINE;
        }
        break;
      case CHUNK_SIZE:
        out.write(i);
        if((byte)i == CR){
          _status = CHUNK_SIZE_CR;
        }else{
          _chunkSizeSb.append((char)i);
        }
        break;
      case CHUNK_SIZE_CR:
        out.write(i);
        _chunkSize = Integer.parseInt(_chunkSizeSb.toString(),16);
        _chunkSizeSb = new StringBuffer(4);
        if((byte)i == LF){
          if(_chunkSize > 0){
            _status = CHUNK_BODY;
          }else{
            //last chunk had been written.
            _status = START_TRAILER;
          }
        }
        break;
      case CHUNK_BODY:
        out.write(i);
        _chunkSize -= 1;
        if(_chunkSize == -2){ // chunk-data + CRLF
          _status = CHUNK_SIZE;
        }
        break;
      case START_TRAILER:
        out.write(i);
        if((byte)i == CR){
          _status = LAST_CR;
        }else{
          _status = IN_TRAILER;
        }
        break;
      case IN_TRAILER:
        out.write(i);
        if((byte)i == CR){
          _status = TRAILER_CR;
        }
        break;
      case TRAILER_CR:
        out.write(i);
        if((byte)i == LF){
          _status = START_TRAILER;
        }
        break;
      case LAST_CR:
        out.write(i);
        if((byte)i == LF){
          _status = START_LINE;
        }
        break;
    }
  }

  /**
  * RXgN^Ŏw肵\Pbg瓾o͐Xg[ɃoCg bytes
  * ̃f[^ށB݂̍ہAȉ
  * lB<p/>
  * RXgN^HTTPwb_̒ǉw肵ꍇ:<br/>
  * ݎAHTTPSTART LINEIɒǉwb_ށB<p/>
  * RXgN^HTTPwb_̍폜w肵ꍇ:<br/>
  * ݃f[^폜Ώۃwb_̈ꕔ̏ꍇ͏܂ȂB
  * @param bytes ރoCgB
  * @exception o̓G[ꍇB
  */
  public void write(byte[] bytes) throws IOException{
    write(bytes, 0, bytes.length);
  }

  /**
  * RXgN^Ŏw肵\Pbg瓾o͐Xg[ɃoCg bytes
  * ̃ItZbg off  oCg len ̃f[^ށB݂̍ہAȉ
  * lB<p/>
  * RXgN^HTTPwb_̒ǉw肵ꍇ:<br/>
  * ݎAHTTPSTART LINEIɒǉwb_ށB<p/>
  * RXgN^HTTPwb_̍폜w肵ꍇ:<br/>
  * ݃f[^폜Ώۃwb_̈ꕔ̏ꍇ͏܂ȂB
  * @param bytes ރoCgB
  * @param off ރoCg bytes ̊JnItZbgB
  * @param len  ރoCgB
  * @exception o̓G[ꍇB
  */
  public void write(byte[] bytes, int off, int len) throws IOException{
    for(int i=off; i<len; i++){
      write(bytes[i]);
    }
  }

  private void checkDeleteHeaders() throws IOException{
    byte[] header = _header.toByteArray();
    String check = new String(_header.toByteArray(), HEADER_ENC);
    boolean writeHeader = true;
    if(_delHttpHeaders != null){
      for(int i=0; i<_delHttpHeaders.length; i++){
        if(check.startsWith(_delHttpHeaders[i])){
          writeHeader = false;
          break;
        }
      }
    }
    if(writeHeader){
      out.write(header);
    }
    _header.reset();
  }

  private void checkBodyLength() throws IOException{
    byte[] b = _header.toByteArray();
    String header = new String(b, HEADER_ENC).trim();
    if(header.startsWith("Transfer-Encoding") && header.endsWith("chunked")){
      _chunkedEnable = true;
    }else if(!_chunkedEnable && header.startsWith("Content-Length")){
      _bodyRemainLength = Integer.parseInt(header.split(":")[1].trim());
    }
  }

  private void writeHeaders() throws IOException{
    if(_httpHeaders == null) return;
    for(int i=0; i< _httpHeaders.length; i++){
      String replaced = _httpHeaders[i];
      if(_httpHeaders[i].indexOf(KW_TIME) >= 0){
        replaced = replaced.replaceFirst(KW_TIME, Long.toString(System.currentTimeMillis()));
      }
      if(_httpHeaders[i].indexOf(KW_DATE) >= 0){
        replaced = replaced.replaceFirst(KW_DATE, new Date().toGMTString());
      }
      byte[] b = replaced.getBytes(HEADER_ENC);
      out.write(b);
      out.write(HTTP_CRLF);
    }
  }

  private String[] buildHttpHeaders(String[] httpHeaders) throws IOException{
    String ret[] = new String[httpHeaders.length];
    for(int i = 0; i < httpHeaders.length; i++){
      String replace = httpHeaders[i];
      if(httpHeaders[i].indexOf(KW_SOCKET) >= 0){
        replace = replace.replaceFirst(KW_SOCKET, _socket.toString());
      }
      if(httpHeaders[i].indexOf(KW_HOST) >= 0){
        replace = replace.replaceFirst(KW_HOST, _socket.getInetAddress().getHostAddress());
      }
      if(httpHeaders[i].indexOf(KW_PORT) >= 0){
        replace = replace.replaceFirst(KW_PORT, Integer.toString(_socket.getPort()));
      }
      if(httpHeaders[i].indexOf(KW_LOCALHOST) >= 0){
        replace = replace.replaceFirst(KW_LOCALHOST, _socket.getLocalAddress().getHostAddress());
      }
      if(httpHeaders[i].indexOf(KW_LOCALPORT) >= 0){
        replace = replace.replaceFirst(KW_LOCALPORT, Integer.toString(_socket.getLocalPort()));
      }
      ret[i] = replace;
    }
    return ret;
  }

}
